"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _invariant = _interopRequireDefault(require("invariant"));

var _react = _interopRequireWildcard(require("react"));

var _Input = _interopRequireDefault(require("./Input"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

// interface DateTimePickerInput extends InputProps {
//   format: RequiredDateMethods,
//   editing: boolean,
//   editFormat: CustomPropTypes.dateFormat,
//   parse: (),
//   value: PropTypes.instanceOf(Date),
//   onChange: PropTypes.func.isRequired,
//   onBlur: PropTypes.func,
//   localizer: PropTypes.object.isRequired,
//   disabled: CustomPropTypes.disabled,
//   readOnly: CustomPropTypes.disabled,
// }
const DateTimePickerInput = _react.default.forwardRef((_ref, ref) => {
  let {
    value,
    editing,
    editFormat,
    format,
    localizer,
    parse,
    onChange,
    onBlur,
    disabled,
    readOnly
  } = _ref,
      props = _objectWithoutProperties(_ref, ["value", "editing", "editFormat", "format", "localizer", "parse", "onChange", "onBlur", "disabled", "readOnly"]);

  const needsFlush = (0, _react.useRef)(false);
  const nextTextValue = (0, _react.useMemo)(() => value instanceof Date && isValid(value) ? localizer.formatDate(value, format) : '', [value, localizer, editing && editFormat ? editFormat : format]);
  const lastValueFromProps = (0, _react.useRef)(nextTextValue);
  const [textValue, setTextValue] = (0, _react.useState)(nextTextValue);

  if (lastValueFromProps.current !== nextTextValue) {
    lastValueFromProps.current = nextTextValue;
    setTextValue(nextTextValue);
  }

  const parseStringInput = string => {
    (0, _invariant.default)(parse || format || editFormat, 'React Widgets: there are no specified `parse` formats provided and the `format` prop is a function. ' + 'the DateTimePicker is unable to parse `%s` into a dateTime, ' + 'please provide either a parse function or localizer compatible `format` prop', string);
    let date;
    let checkFormats = [format, editFormat];

    if (typeof parse == 'function') {
      date = parse(string);
      if (date) return date;
    } else {
      // parse is a string format or array of string formats
      checkFormats = checkFormats.concat(parse).filter(Boolean);
    }

    for (let i = 0; i < checkFormats.length; i++) {
      date = localizer.parseDate(string, checkFormats[i]);
      if (date) return date;
    }

    return null;
  };

  const handleBlur = event => {
    if (onBlur) onBlur(event);

    if (needsFlush.current) {
      let date = parseStringInput(event.target.value);
      const dateIsInvalid = event.target.value != '' && date == null;

      if (dateIsInvalid) {
        setTextValue('');
      }

      needsFlush.current = false;
      onChange(date, event.target.value);
    }
  };

  const handleChange = ({
    target: {
      value
    }
  }) => {
    needsFlush.current = true;
    setTextValue(value);
  };

  return _react.default.createElement(_Input.default, Object.assign({}, props, {
    type: "text",
    ref: ref,
    className: "rw-widget-input",
    value: textValue,
    disabled: disabled,
    readOnly: readOnly,
    onChange: handleChange,
    onBlur: handleBlur
  }));
});

DateTimePickerInput.displayName = 'DateTimePickerInput';
var _default = DateTimePickerInput;
exports.default = _default;

function isValid(d) {
  return !isNaN(d.getTime());
}