"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classnames = _interopRequireDefault(require("classnames"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _react = _interopRequireWildcard(require("react"));

var _uncontrollable = require("uncontrollable");

var _BaseListbox = _interopRequireDefault(require("./BaseListbox"));

var _ListOption = _interopRequireDefault(require("./ListOption"));

var _ListOptionGroup = _interopRequireDefault(require("./ListOptionGroup"));

var _ListboxContext = require("./ListboxContext");

var _messages = require("./messages");

var CustomPropTypes = _interopRequireWildcard(require("./util/PropTypes"));

var Props = _interopRequireWildcard(require("./util/Props"));

var _2 = require("./util/_");

var _getAccessors = require("./util/getAccessors");

var _widgetHelpers = require("./util/widgetHelpers");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const propTypes = {
  data: _propTypes.default.array,
  dataKey: CustomPropTypes.accessor,
  textField: CustomPropTypes.accessor,
  onSelect: _propTypes.default.func,
  onMove: _propTypes.default.func,
  onHoverOption: _propTypes.default.func,
  activeId: _propTypes.default.string,
  optionComponent: CustomPropTypes.elementType,
  renderItem: _propTypes.default.func,
  renderGroup: _propTypes.default.func,
  focusedItem: _propTypes.default.any,
  selectedItem: _propTypes.default.any,
  searchTerm: _propTypes.default.string,
  disabled: CustomPropTypes.disabled.acceptsArray,
  messages: _propTypes.default.shape({
    emptyList: _propTypes.default.func.isRequired
  })
};

const isGroup = item => typeof item === 'object' && item != null && '__isGroup' in item;

const defaultProps = {
  onSelect: () => {},
  data: [],
  optionComponent: _ListOption.default
};

function useFlattenedData(data, groupBy) {
  return (0, _react.useMemo)(() => {
    if (!groupBy) return data;
    const flatData = [];
    let keys = [];
    let grouped = (0, _2.groupBySortedKeys)(groupBy, data, keys);
    keys.forEach(group => {
      flatData.push({
        __isGroup: group
      }, ...grouped[group]);
    }, []);
    return flatData;
  }, [data, groupBy]);
}

const Listbox = _react.default.forwardRef(function Listbox(_ref, outerRef) {
  let {
    multiple = false,
    focusedItem: propsFocusedItem,
    data,
    value,
    defaultValue,
    onChange,
    className,
    messages,
    disabled,
    renderItem,
    textField,
    dataKey,
    renderGroup,
    activeId,
    searchTerm,
    groupBy,
    bordered = true,
    optionComponent: Option = _ListOption.default,
    onKeyDown
  } = _ref,
      props = _objectWithoutProperties(_ref, ["multiple", "focusedItem", "data", "value", "defaultValue", "onChange", "className", "messages", "disabled", "renderItem", "textField", "dataKey", "renderGroup", "activeId", "searchTerm", "groupBy", "bordered", "optionComponent", "onKeyDown"]);

  const [focusedItem, setFocusedItem] = (0, _uncontrollable.useUncontrolledProp)(propsFocusedItem, null, _ => {});
  const [rawValue, handleChange] = (0, _uncontrollable.useUncontrolledProp)(value, defaultValue, onChange);
  const accessors = (0, _getAccessors.useAccessors)(textField, dataKey);
  const currentValue = (0, _2.makeArray)(rawValue);
  const dataItems = (0, _react.useMemo)(() => currentValue.map(item => accessors.findOrSelf(data, item)), [data, currentValue, accessors]);
  const ref = (0, _react.useRef)();
  const disabledItems = (0, _2.toItemArray)(disabled);
  const {
    emptyList
  } = (0, _messages.useMessagesWithDefaults)(messages);
  const flatData = useFlattenedData(data, groupBy);
  const list = (0, _ListboxContext.getList)(flatData, accessors.text, disabledItems);

  const handleSelect = (dataItem, event) => {
    if (multiple === false) {
      const handler = handleChange;
      handler(dataItem, {
        lastValue: value,
        originalEvent: event
      });
      return;
    }

    const handler = handleChange;
    const checked = dataItems.indexOf(dataItem) > -1;
    handler(checked ? dataItems.filter(d => d !== dataItem) : [...dataItems, dataItem], {
      dataItem,
      lastValue: value,
      action: checked ? 'remove' : 'insert',
      originalEvent: event
    });
  };

  const scrollIntoView = (0, _react.useCallback)(() => {
    let list = ref.current;
    if (!list || !focusedItem) return;
    let selectedItem = list.querySelector('[data-rw-focused]');

    if (selectedItem && selectedItem.scrollIntoView) {
      selectedItem.scrollIntoView({
        block: 'nearest',
        inline: 'nearest'
      });
    }
  }, [focusedItem]);
  (0, _react.useLayoutEffect)(scrollIntoView, [scrollIntoView]);
  (0, _ListboxContext.useClearListOptions)();
  let elementProps = Props.pickElementProps(props);
  (0, _react.useImperativeHandle)(outerRef, () => ({
    scrollIntoView
  }), [scrollIntoView]);

  const handleKeyDown = e => {
    let {
      key,
      shiftKey
    } = e;
    const currentFocused = focusedItem || currentValue[currentValue.length - 1];
    (0, _widgetHelpers.notify)(onKeyDown, [e]);
    if (e.defaultPrevented) return;

    if (key === 'End' && !shiftKey) {
      e.preventDefault();
      setFocusedItem(list.last());
    } else if (key === 'Home' && !shiftKey) {
      e.preventDefault();
      setFocusedItem(list.first());
    } else if (key === 'Enter') {
      e.preventDefault();
      if (focusedItem) handleSelect(focusedItem, e);
    } else if (key === 'ArrowDown') {
      e.preventDefault();
      setFocusedItem(list.next(currentFocused));
    } else if (key === 'ArrowUp') {
      e.preventDefault();
      setFocusedItem(list.prev(currentFocused));
    }
  };

  return _react.default.createElement(_BaseListbox.default, Object.assign({
    ref: ref
  }, elementProps, {
    multiple: multiple,
    onKeyDown: handleKeyDown,
    className: (0, _classnames.default)(className, bordered && 'rw-widget-container'),
    emptyListMessage: emptyList()
  }), flatData.map((item, idx) => {
    if (isGroup(item)) return _react.default.createElement(_ListOptionGroup.default, {
      key: `group_${item.__isGroup}`
    }, renderGroup ? renderGroup({
      group: item.__isGroup
    }) : item.__isGroup);
    let isFocused = focusedItem === item;
    const textValue = accessors.text(item);
    const itemIsDisabled = disabledItems.indexOf(item) !== -1;
    const itemIsSelected = dataItems.indexOf(item) !== -1;
    return _react.default.createElement(Option, {
      dataItem: item,
      key: `item_${idx}`,
      activeId: activeId,
      focused: isFocused,
      onSelect: handleSelect,
      disabled: itemIsDisabled,
      selected: itemIsSelected
    }, renderItem ? renderItem({
      item,
      searchTerm,
      index: idx,
      text: textValue,
      // TODO: probably remove
      value: accessors.value(item),
      disabled: itemIsDisabled
    }) : textValue);
  }));
});

Listbox.displayName = 'Listbox';

var _default = Object.assign(Listbox, {
  defaultProps,
  propTypes
});

exports.default = _default;