"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getList = getList;
exports.useOptionList = useOptionList;
exports.useClearListOptions = useClearListOptions;
exports.useListOption = useListOption;
exports.OptionsContext = void 0;

var _react = _interopRequireWildcard(require("react"));

var _Filter = require("./util/Filter");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const EMPTY_VALUE = {};

const OptionsContext = _react.default.createContext(null);

exports.OptionsContext = OptionsContext;

function getList(options, textAccessor, disabledItems) {
  const isDisabled = item => disabledItems.indexOf(item) !== -1;

  let moveNext = (item, word) => isDisabled(item) || word && !_Filter.presets.startsWith(textAccessor(item).toLowerCase(), word.toLowerCase());

  const list = {
    first: () => list.next(EMPTY_VALUE),
    last: () => {
      const item = options[options.length - 1];
      return isDisabled(item) ? list.prev(item) : item;
    },

    prev(item, word) {
      let nextIdx = Math.max(0, options.indexOf(item)) - 1;

      while (nextIdx > -1 && moveNext(options[nextIdx], word)) nextIdx--;

      if (nextIdx >= 0) return options[nextIdx];
      return isDisabled(item) ? undefined : item;
    },

    next(item, word) {
      let nextIdx = options.indexOf(item) + 1;

      while (nextIdx < options.length && moveNext(options[nextIdx], word)) nextIdx++;

      if (nextIdx < options.length) return options[nextIdx];
      return isDisabled(item) ? undefined : item;
    },

    nextEnabled: item => isDisabled(item) ? list.next(item) : item
  };
  return list;
}

function useOptionList(textAccessor, disabledItems) {
  const options = (0, _react.useMemo)(() => [], []);
  const list = getList(options, textAccessor, disabledItems);
  return [(0, _react.useMemo)(() => ({
    options
  }), [options]), list];
}

function useClearListOptions(providedContext) {
  const parent = (0, _react.useContext)(OptionsContext);
  const ctx = providedContext || parent;
  (0, _react.useLayoutEffect)(() => {
    if (ctx) ctx.options.length = 0;
  });
}

function useListOption(dataItem) {
  const ctx = (0, _react.useContext)(OptionsContext);
  (0, _react.useEffect)(() => {
    if (!ctx) return;
    const idx = ctx.options.length;
    ctx.options.push(dataItem);
    return () => {
      if (ctx.options[idx] === dataItem) {
        ctx.options.splice(idx, 1);
      }
    };
  });
}