"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classnames = _interopRequireDefault(require("classnames"));

var _closest = _interopRequireDefault(require("dom-helpers/closest"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _react = _interopRequireWildcard(require("react"));

var _uncontrollable = require("uncontrollable");

var _AddToListOption = _interopRequireWildcard(require("./AddToListOption"));

var _Icon = require("./Icon");

var _Listbox = _interopRequireDefault(require("./Listbox"));

var _ListboxContext = require("./ListboxContext");

var _MultiselectInput = _interopRequireDefault(require("./MultiselectInput"));

var _MultiselectTagList = _interopRequireDefault(require("./MultiselectTagList"));

var _Popup = _interopRequireDefault(require("./Popup"));

var _Select = _interopRequireDefault(require("./Select"));

var _Widget = _interopRequireDefault(require("./Widget"));

var _WidgetPicker = _interopRequireDefault(require("./WidgetPicker"));

var _messages = require("./messages");

var _A11y = require("./util/A11y");

var Filter = _interopRequireWildcard(require("./util/Filter"));

var CustomPropTypes = _interopRequireWildcard(require("./util/PropTypes"));

var _2 = require("./util/_");

var _canShowCreate = _interopRequireDefault(require("./util/canShowCreate"));

var _getAccessors = require("./util/getAccessors");

var _hooks = require("./util/hooks");

var _useFocusManager = _interopRequireDefault(require("./util/useFocusManager"));

var _widgetHelpers = require("./util/widgetHelpers");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const ENTER = 13;
const INSERT = 'insert';
const REMOVE = 'remove';

let propTypes = _objectSpread({}, Filter.propTypes, {
  data: _propTypes.default.array,
  //-- controlled props --
  value: _propTypes.default.array,

  /**
   * @type {function (
   *  dataItems: ?any[],
   *  metadata: {
   *    dataItem: any,
   *    action: 'insert' | 'remove',
   *    originalEvent: SyntheticEvent,
   *    lastValue: ?any[],
   *    searchTerm: ?string
   *  }
   * ): void}
   */
  onChange: _propTypes.default.func,
  searchTerm: _propTypes.default.string,

  /**
   * @type {function (
   *  searchTerm: ?string,
   *  metadata: {
   *    action: 'clear' | 'input',
   *    lastSearchTerm: ?string,
   *    originalEvent: SyntheticEvent,
   *  }
   * ): void}
   */
  onSearch: _propTypes.default.func,
  open: _propTypes.default.bool,
  handleOpen: _propTypes.default.func,
  //-------------------------------------------
  dataKey: CustomPropTypes.accessor,
  textField: CustomPropTypes.accessor,
  renderTagValue: _propTypes.default.func,

  /**
   * Control the rendering of the outer tag component, including the delete button. To control just hte tag label, `use tagComponent` instead
   */
  renderTag: CustomPropTypes.elementType,
  renderListItem: _propTypes.default.func,
  renderListGroup: _propTypes.default.func,
  groupBy: CustomPropTypes.accessor,
  allowCreate: _propTypes.default.oneOf([true, false, 'onFilter']),

  /**
   *
   * @type { (dataItem: ?any, metadata: { originalEvent: SyntheticEvent }) => void }
   */
  onSelect: _propTypes.default.func,

  /**
   * @type { (searchTerm: string) => void }
   */
  onCreate: _propTypes.default.func,
  busy: _propTypes.default.bool,

  /** Specify the element used to render the select (down arrow) icon. */
  selectIcon: _propTypes.default.node,

  /** Specify the element used to render tag clear icons. */
  clearTagIcon: _propTypes.default.node,

  /** Specify the element used to render the busy indicator */
  busySpinner: _propTypes.default.node,
  dropUp: _propTypes.default.bool,
  popupTransition: CustomPropTypes.elementType,

  /** Adds a css class to the input container element. */
  containerClassName: _propTypes.default.string,
  inputProps: _propTypes.default.object,
  listProps: _propTypes.default.object,
  autoFocus: _propTypes.default.bool,
  placeholder: _propTypes.default.string,

  /** Continue to show the input placeholder even if tags are selected */
  showPlaceholderWithValues: _propTypes.default.bool,
  disabled: CustomPropTypes.disabled.acceptsArray,
  readOnly: CustomPropTypes.disabled,
  isRtl: _propTypes.default.bool,
  messages: _propTypes.default.shape({
    open: CustomPropTypes.message,
    emptyList: CustomPropTypes.message,
    emptyFilter: CustomPropTypes.message,
    createOption: CustomPropTypes.message,
    tagsLabel: CustomPropTypes.message,
    selectedItems: CustomPropTypes.message,
    noneSelected: CustomPropTypes.message,
    removeLabel: CustomPropTypes.message
  })
});

const EMPTY_ARRAY = [];

function useMultiselectData(value = EMPTY_ARRAY, data, filter, searchTerm, accessors) {
  data = (0, _react.useMemo)(() => data.filter(i => !value.some(v => accessors.matches(i, v))), [data, value, accessors]);
  return [(0, _hooks.useFilteredData)(data, filter, searchTerm, accessors.text), data.length];
}

/**
 * ---
 * shortcuts:
 *   - { key: left arrow, label: move focus to previous tag }
 *   - { key: right arrow, label: move focus to next tag }
 *   - { key: delete, deselect focused tag }
 *   - { key: backspace, deselect next tag }
 *   - { key: alt + up arrow, label: close Multiselect }
 *   - { key: down arrow, label: open Multiselect, and move focus to next item }
 *   - { key: up arrow, label: move focus to previous item }
 *   - { key: home, label: move focus to first item }
 *   - { key: end, label: move focus to last item }
 *   - { key: enter, label: select focused item }
 *   - { key: ctrl + enter, label: create new tag from current searchTerm }
 *   - { key: any key, label: search list for item starting with key }
 * ---
 *
 * A select listbox alternative.
 *
 * @public
 */
const Multiselect = _react.default.forwardRef(function Multiselect(_ref, outerRef) {
  let {
    dataKey,
    textField,
    autoFocus,
    id,
    value,
    defaultValue = [],
    onChange,
    open,
    defaultOpen = false,
    onToggle,
    searchTerm,
    defaultSearchTerm = '',
    onSearch,
    filter = 'startsWith',
    allowCreate = false,
    isRtl,
    className,
    containerClassName,
    placeholder,
    busy,
    disabled,
    readOnly,
    selectIcon = _Icon.caretDown,
    clearTagIcon = _Icon.times,
    busySpinner,
    dropUp,
    tabIndex,
    popupTransition,
    showPlaceholderWithValues = false,
    onSelect,
    onCreate,
    onKeyDown,
    onBlur,
    onFocus,
    inputProps,
    listProps,
    renderListItem,
    renderListGroup,
    renderTagValue,
    optionComponent,
    tagOptionComponent,
    listComponent: List = _Listbox.default,
    data: rawData = [],
    messages: userMessages,
    groupBy: _
  } = _ref,
      elementProps = _objectWithoutProperties(_ref, ["dataKey", "textField", "autoFocus", "id", "value", "defaultValue", "onChange", "open", "defaultOpen", "onToggle", "searchTerm", "defaultSearchTerm", "onSearch", "filter", "allowCreate", "isRtl", "className", "containerClassName", "placeholder", "busy", "disabled", "readOnly", "selectIcon", "clearTagIcon", "busySpinner", "dropUp", "tabIndex", "popupTransition", "showPlaceholderWithValues", "onSelect", "onCreate", "onKeyDown", "onBlur", "onFocus", "inputProps", "listProps", "renderListItem", "renderListGroup", "renderTagValue", "optionComponent", "tagOptionComponent", "listComponent", "data", "messages", "groupBy"]);

  let [currentValue, handleChange] = (0, _uncontrollable.useUncontrolledProp)(value, defaultValue, onChange);
  const [currentOpen, handleOpen] = (0, _uncontrollable.useUncontrolledProp)(open, defaultOpen, onToggle);
  const [currentSearch, handleSearch] = (0, _uncontrollable.useUncontrolledProp)(searchTerm, defaultSearchTerm, onSearch);
  const ref = (0, _react.useRef)(null);
  const inputRef = (0, _react.useRef)(null);
  const listRef = (0, _react.useRef)(null);
  const inputId = (0, _widgetHelpers.useInstanceId)(id, '_input');
  const tagsId = (0, _widgetHelpers.useInstanceId)(id, '_taglist');
  const notifyId = (0, _widgetHelpers.useInstanceId)(id, '_notify_area');
  const listId = (0, _widgetHelpers.useInstanceId)(id, '_listbox');
  const createId = (0, _widgetHelpers.useInstanceId)(id, '_createlist_option');
  const activeTagId = (0, _widgetHelpers.useInstanceId)(id, '_taglist_active_tag');
  const activeOptionId = (0, _widgetHelpers.useInstanceId)(id, '_listbox_active_option');
  const accessors = (0, _getAccessors.useAccessors)(textField, dataKey);
  const messages = (0, _messages.useMessagesWithDefaults)(userMessages);
  const toggle = (0, _hooks.useDropodownToggle)(currentOpen, handleOpen);
  const isDisabled = disabled === true;
  const disabledItems = (0, _2.toItemArray)(disabled);
  const isReadOnly = !!readOnly;
  const [focusEvents, focused] = (0, _useFocusManager.default)(ref, {
    disabled: isDisabled,
    onBlur,
    onFocus
  }, {
    didHandle(focused, event) {
      if (focused) return focus();
      toggle.close();
      clearSearch(event);
      setFocusedTag(null);
    }

  });
  const dataItems = (0, _react.useMemo)(() => currentValue.map(item => accessors.findOrSelf(rawData, item)), [rawData, currentValue, accessors]);
  const [data, lengthWithoutValues] = useMultiselectData(currentValue, rawData, currentOpen ? filter : false, currentSearch, accessors);
  const [listContext, list] = (0, _ListboxContext.useOptionList)(accessors.text, []);
  const [tagListContext, tagList] = (0, _ListboxContext.useOptionList)(accessors.text, disabledItems);
  const [focusedItem, setFocusedItem] = (0, _react.useState)();
  const [focusedTag, setFocusedTag] = (0, _react.useState)();

  if (focusedTag && dataItems.indexOf(focusedTag) === -1) {
    setFocusedTag(null);
  }

  const showCreateOption = (0, _canShowCreate.default)(allowCreate, {
    searchTerm: currentSearch,
    data,
    dataItems,
    accessors
  });
  /**
   * Update aria when it changes on update
   */

  (0, _react.useEffect)(() => {
    let active;
    if (!currentOpen) active = focusedTag ? activeTagId : '';else if (focusedItem || showCreateOption) active = activeOptionId;
    (0, _A11y.setActiveDescendant)(inputRef.current, active, currentOpen);
  }, [currentOpen, focusedItem]);
  /**
   * Event Handlers
   */

  const handleDelete = (dataItem, event) => {
    if (isDisabled || readOnly) return;
    focus();
    change(dataItem, event, REMOVE);
  };

  const deletingRef = (0, _react.useRef)(false);

  const handleSearchKeyDown = e => {
    if (e.key === 'Backspace' && e.currentTarget.value && !deletingRef.current) deletingRef.current = true;
  };

  const handleSearchKeyUp = e => {
    if (e.key === 'Backspace' && deletingRef.current) {
      deletingRef.current = false;
    }
  };

  const handleInputChange = e => {
    search(e.target.value, e, 'input');
    toggle.open();
  };

  const handleClick = ({
    target
  }) => {
    focus();

    if ((0, _closest.default)(target, '.rw-select') && currentOpen) {
      toggle.close();
    } else toggle.open();
  };

  const handleDoubleClick = () => {
    if (!inputRef.current) return;
    focus();
    if (inputRef.current) inputRef.current.select();
  };

  const handleSelect = (dataItem, originalEvent) => {
    if (dataItem === undefined || dataItem === _AddToListOption.CREATE_OPTION) {
      handleCreate(currentSearch, originalEvent);
      return;
    }

    (0, _widgetHelpers.notify)(onSelect, [dataItem, {
      originalEvent
    }]);
    change(dataItem, originalEvent, INSERT);
    focus();
  };

  const handleCreate = (_, event) => {
    (0, _widgetHelpers.notify)(onCreate, [currentSearch]);
    clearSearch(event);
    focus();
  };

  const handleKeyDown = event => {
    let {
      key,
      keyCode,
      altKey,
      ctrlKey
    } = event;
    (0, _widgetHelpers.notify)(onKeyDown, [event]);

    const setFocused = (listItem, tag) => {
      if (listItem || tag) setFocusedItem(listItem || null);
      if (listItem || tag) setFocusedTag(tag || null);
    };

    if (event.defaultPrevented) return;

    if (key === 'ArrowDown') {
      event.preventDefault();

      if (!currentOpen) {
        toggle.open();
        return;
      }

      setFocused(list.next(focusedItem));
    } else if (key === 'ArrowUp' && (currentOpen || altKey)) {
      event.preventDefault();

      if (altKey) {
        toggle.close();
        return;
      }

      setFocused(list.prev(focusedItem));
    } else if (key === 'End') {
      event.preventDefault();
      if (currentOpen) setFocused(list.last());else setFocused(tagList.last());
    } else if (key === 'Home') {
      event.preventDefault();
      if (currentOpen) setFocused(list.first());else setFocused(tagList.first());
    } else if (currentOpen && keyCode === ENTER) {
      // using keyCode to ignore enter for japanese IME
      event.preventDefault();

      if (ctrlKey && showCreateOption) {
        return handleCreate(currentSearch, event);
      }

      handleSelect(focusedItem, event);
    } else if (key === 'Escape') {
      if (currentOpen) toggle.close();else setFocusedTag(null); //
    } else if (!currentSearch && !deletingRef.current) {
      //
      if (key === 'ArrowLeft') {
        setFocusedTag(tagList.prev(focusedTag) || tagList.last());
      } else if (key === 'ArrowRight' && focusedTag) {
        let nextTag = tagList.next(focusedTag);
        setFocusedTag(nextTag === focusedTag ? null : nextTag); //
      } else if (key === 'Delete' && disabledItems.indexOf(focusedItem) === -1) {
        handleDelete(focusedTag, event); //
      } else if (key === 'Backspace') {
        handleDelete(tagList.last(), event); //
      } else if (key === ' ' && !currentOpen) {
        event.preventDefault();
        toggle.open();
      }
    }
  };
  /**
   * Methods
   */


  function change(dataItem, originalEvent, action) {
    let nextDataItems = dataItems;

    switch (action) {
      case INSERT:
        nextDataItems = nextDataItems.concat(dataItem);
        break;

      case REMOVE:
        nextDataItems = nextDataItems.filter(d => d !== dataItem);
        break;
    }

    handleChange(nextDataItems, {
      action,
      dataItem,
      originalEvent,
      searchTerm: currentSearch,
      lastValue: currentValue
    });
    clearSearch(originalEvent);
  }

  function clearSearch(originalEvent) {
    search('', originalEvent, 'clear');
  }

  function search(nextSearchTerm, originalEvent, action = 'input') {
    if (nextSearchTerm !== currentSearch) handleSearch(nextSearchTerm, {
      action,
      originalEvent,
      lastSearchTerm: currentSearch
    });
  }

  function focus() {
    if (inputRef.current) inputRef.current.focus();
  }
  /**
   * Render
   */


  (0, _react.useImperativeHandle)(outerRef, () => ({
    focus
  }));
  let shouldRenderPopup = (0, _widgetHelpers.useFirstFocusedRender)(focused, currentOpen);
  let itemLabels = dataItems.map(item => accessors.text(item));
  let shouldRenderTags = !!dataItems.length;
  let inputOwns = `${listId} ${notifyId} ` + (shouldRenderTags ? tagsId : '') + (showCreateOption ? createId : '');
  return _react.default.createElement(_Widget.default, Object.assign({}, elementProps, {
    open: currentOpen,
    isRtl: isRtl,
    dropUp: dropUp,
    focused: focused,
    disabled: isDisabled,
    readOnly: isReadOnly,
    onKeyDown: handleKeyDown
  }, focusEvents, {
    className: (0, _classnames.default)(className, 'rw-multiselect')
  }), _react.default.createElement("span", {
    id: notifyId,
    role: "status",
    className: "rw-sr",
    "aria-live": "assertive",
    "aria-atomic": "true",
    "aria-relevant": "all"
  }, focused && (dataItems.length ? messages.selectedItems(itemLabels) : messages.noneSelected())), _react.default.createElement(_WidgetPicker.default, {
    onClick: handleClick,
    onTouchEnd: handleClick,
    onDoubleClick: handleDoubleClick,
    className: (0, _classnames.default)(containerClassName, 'rw-widget-input')
  }, _react.default.createElement(_ListboxContext.OptionsContext.Provider, {
    value: tagListContext
  }, _react.default.createElement(_MultiselectTagList.default, {
    id: tagsId,
    activeId: activeTagId,
    textAccessor: accessors.text,
    dataKeyAccessor: accessors.value,
    clearTagIcon: clearTagIcon,
    label: messages.tagsLabel(),
    value: dataItems,
    disabled: disabledItems,
    focusedItem: focusedTag,
    onDelete: handleDelete,
    tagOptionComponent: tagOptionComponent,
    renderTagValue: renderTagValue
  }, _react.default.createElement(_MultiselectInput.default, Object.assign({}, inputProps, {
    "aria-haspopup": true,
    role: "listbox",
    autoFocus: autoFocus,
    tabIndex: tabIndex || 0,
    "aria-expanded": !!currentOpen,
    "aria-busy": !!busy,
    "aria-owns": inputOwns,
    value: currentSearch,
    disabled: isDisabled,
    readOnly: isReadOnly,
    placeholder: (currentValue.length && !showPlaceholderWithValues ? '' : placeholder) || '',
    onKeyDown: handleSearchKeyDown,
    onKeyUp: handleSearchKeyUp,
    onChange: handleInputChange,
    ref: inputRef
  })))), _react.default.createElement(_Select.default, {
    busy: busy,
    "aria-hidden": "true",
    role: "presentational",
    spinner: busySpinner,
    icon: focused ? selectIcon : null,
    disabled: isDisabled || isReadOnly
  })), _react.default.createElement(_ListboxContext.OptionsContext.Provider, {
    value: listContext
  }, shouldRenderPopup && _react.default.createElement(_Popup.default, {
    dropUp: dropUp,
    open: currentOpen,
    transition: popupTransition,
    onEntering: () => listRef.current.scrollIntoView()
  }, _react.default.createElement(List, Object.assign({}, listProps, {
    id: listId,
    data: data,
    tabIndex: -1,
    bordered: false,
    disabled: disabled,
    activeId: activeOptionId,
    searchTerm: currentSearch,
    textField: textField,
    dataKey: dataKey,
    renderItem: renderListItem,
    renderGroup: renderListGroup,
    optionComponent: optionComponent,
    focusedItem: focusedItem,
    onChange: (d, meta) => handleSelect(d, meta.originalEvent),
    "aria-live": "polite",
    "aria-labelledby": inputId,
    "aria-hidden": !currentOpen,
    ref: listRef,
    messages: {
      emptyList: lengthWithoutValues ? messages.emptyFilter : messages.emptyList
    }
  })), showCreateOption && _react.default.createElement(_AddToListOption.default, {
    id: createId,
    onSelect: handleCreate,
    focused: focusedItem === _AddToListOption.CREATE_OPTION
  }, messages.createOption(currentValue, currentSearch)))));
});

Multiselect.displayName = 'Multiselect';
Multiselect.propTypes = propTypes; // Multiselect.defaultProps = defaultProps

var _default = Multiselect;
exports.default = _default;