"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useActiveDescendant = exports.setActiveDescendant = void 0;

var _react = require("react");

var _reactDom = require("react-dom");

/**
 * Do a "hard" set on the aria, so that it's always announced
 * even if the id hasn't changed, this saves us from having to have a different id
 * per item.
 */
const setActiveDescendant = (ref, activeId, visible) => {
  if (!ref) return;
  const node = (0, _reactDom.findDOMNode)(ref);
  node.removeAttribute('aria-activedescendant');
  if (visible) node.setAttribute('aria-activedescendant', activeId);
};

exports.setActiveDescendant = setActiveDescendant;

const useActiveDescendant = (ref, id, visible, deps) => {
  (0, _react.useEffect)(() => {
    setActiveDescendant(ref.current, id, visible);
  }, [visible, ...deps]);
};

exports.useActiveDescendant = useActiveDescendant;