"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toItemArray = toItemArray;
exports.isShallowEqual = isShallowEqual;
exports.chunk = chunk;
exports.groupBySortedKeys = groupBySortedKeys;
exports.has = exports.makeArray = void 0;

var _warning = _interopRequireDefault(require("warning"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toItemArray(a) {
  if (Array.isArray(a)) return a;
  return [];
}

const makeArray = obj => obj == null ? [] : [].concat(obj);

exports.makeArray = makeArray;

const has = (o, key) => o ? Object.prototype.hasOwnProperty.call(o, key) : false;

exports.has = has;

function isShallowEqual(a, b) {
  if (a === b) return true;
  if (a instanceof Date && b instanceof Date) return +a === +b;
  if (typeof a !== 'object' && typeof b !== 'object') return a === b;
  if (typeof a !== typeof b) return false;
  if (a == null || b == null) return false; // if they were both null we wouldn't be here

  let keysA = Object.keys(a);
  let keysB = Object.keys(b);
  if (keysA.length !== keysB.length) return false;

  for (let i = 0; i < keysA.length; i++) if (!has(b, keysA[i]) || a[keysA[i]] !== b[keysA[i]]) return false;

  return true;
}

function chunk(array, chunkSize) {
  let index = 0;
  let length = array ? array.length : 0;
  let result = [];
  chunkSize = Math.max(+chunkSize || 1, 1);

  while (index < length) result.push(array.slice(index, index += chunkSize));

  return result;
}

function groupBySortedKeys(groupBy, data, keys = []) {
  const iter = typeof groupBy === 'function' ? groupBy : item => item[groupBy];
  (0, _warning.default)(typeof groupBy !== 'string' || !data.length || has(data[0], groupBy), `[React Widgets] You seem to be trying to group this list by a ` + `property \`${groupBy}\` that doesn't exist in the dataset items, this may be a typo`);
  return data.reduce((grps, item) => {
    let group = iter(item);

    if (has(grps, group)) {
      grps[group].push(item);
    } else {
      keys.push(group);
      grps[group] = [item];
    }

    return grps;
  }, {});
}