"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useAutoFocus = useAutoFocus;
exports.useStateFromProp = useStateFromProp;
exports.useDropodownToggle = useDropodownToggle;
exports.useFilteredData = useFilteredData;
exports.useKeyboardNavigationCheck = useKeyboardNavigationCheck;

var _react = require("react");

var _useGlobalListener = _interopRequireDefault(require("@restart/hooks/useGlobalListener"));

var _useStableMemo = _interopRequireDefault(require("@restart/hooks/useStableMemo"));

var _Filter = require("./Filter");

var _widgetHelpers = require("./widgetHelpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function useAutoFocus(autoFocus, ref) {
  (0, _react.useEffect)(() => {
    if (autoFocus && ref.current) ref.current.focus();
  }, []);
}

function useImmediateUpdateEffect(updater, deps) {
  const firstRef = (0, _react.useRef)(true);
  (0, _useStableMemo.default)(() => {
    if (firstRef.current) {
      firstRef.current = false;
      return;
    }

    updater();
  }, deps);
}

function useStateFromProp(propValue) {
  const [focusedItem, setFocusedItem] = (0, _react.useState)(propValue);
  useImmediateUpdateEffect(() => {
    setFocusedItem(propValue);
  }, [propValue]);
  return [focusedItem, setFocusedItem];
}

function useDropodownToggle(isOpen, onToggle) {
  function open() {
    if (!isOpen) (0, _widgetHelpers.notify)(onToggle, [true]);
  }

  function close() {
    if (isOpen) (0, _widgetHelpers.notify)(onToggle, [false]);
  }

  function toggle() {
    if (isOpen) close();else open();
  }

  toggle.open = open;
  toggle.close = close;
  return toggle;
}

function useFilteredData(data, filterer, searchTerm, textAccessor) {
  return (0, _react.useMemo)(() => (0, _Filter.filter)(data, {
    searchTerm,
    filter: filterer,
    textField: textAccessor
  }), [data, _Filter.filter, searchTerm, textAccessor]);
}

function useKeyboardNavigationCheck() {
  const [isNavigatingViaKeyboard, setIsNavigatingViaKeyboard] = (0, _react.useState)(false);
  (0, _useGlobalListener.default)('keydown', ({
    key
  }) => {
    if (key == ' ' || key === 'Tab' || key == 'Enter' || key.indexOf('Arrow') !== -1) {
      setIsNavigatingViaKeyboard(true);
    }
  }); // TODO: use pointerdown once we polyfill it

  (0, _useGlobalListener.default)('mousedown', () => {
    setIsNavigatingViaKeyboard(false);
  });
  return isNavigatingViaKeyboard;
}